var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { toCamelCase } from '../../helpers';
import { serializeAttrValue } from '../serialize';
/**
 * This module create property instance and get/set for all attributes and observedAttributes
 */
var BootstrapClass = /** @class */ (function (_super) {
    __extends(BootstrapClass, _super);
    function BootstrapClass() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * This is re-usable static method, which sets property on the instance prototype based on the given attribute name
     * @param {string} name attribute name
     */
    BootstrapClass.prototype.defineProp = function (name, reflectedAttrToProps) {
        var _this = this;
        if (!name)
            return;
        var propName = toCamelCase(name);
        var _a = reflectedAttrToProps[name], type = _a.type, observe = _a.observe;
        var get = function () {
            var value = _this.getAttribute(name);
            // boolean attribute, then check if attribute exist or not
            if (true.constructor === type && !value)
                return _this.hasAttribute(name);
            // attribute value is defined as Array or Object
            if (type === (Array || Object) && _this["_" + propName]) {
                return _this["_" + propName];
            }
            // return serialized value from the attribute
            if (typeof value === 'string')
                return serializeAttrValue(name, value, type);
        };
        var set = function (value) {
            if (!observe)
                return false;
            var valueAsString = String(value);
            // when value is null or false remove attribute
            if (value == null || valueAsString === 'false')
                return _this.removeAttribute(name);
            // set boolean attribute
            if (valueAsString === 'true')
                return _this.setAttribute(name, '');
            // when value is a empty string then simply set that as attribute value and skip parsing
            if (value === '' || typeof value === 'string') {
                var parsedVal = value === '' ? '' : serializeAttrValue(name, value, type);
                return _this.setAttribute(name, parsedVal);
            }
            // update/create attribute value as string, when it's used for rich data
            if (type === (Array || Object)) {
                var stringifiedValue = JSON.stringify(value);
                _this["_" + propName] = value;
                _this.setAttribute(name, stringifiedValue);
            }
        };
        Object.defineProperty(this, propName, { get: get, set: set, configurable: true });
    };
    /**
     * This loops through all the given attributes, observedAttributes and send it to `defineProp()`
     * @param {object} attributes list of attributes added to the component
     * @param {array} observedAttributes list of observed attributes
     */
    BootstrapClass.prototype.createAttributesToProperties = function (reflectedAttrToProps) {
        var _this = this;
        Object.keys(reflectedAttrToProps).forEach(function (attrName) { return _this.defineProp(attrName, reflectedAttrToProps); });
    };
    /**
     * has attribute
     * @param {string} attribute name
     * @param {object} target DOM element
     */
    BootstrapClass.prototype.hasAttr = function (attr, target) {
        if (target === void 0) { target = this; }
        return target.hasAttribute(attr);
    };
    /**
     * set attribute value
     * @param {string} attribute name
     * @param {object} target DOM element
     */
    BootstrapClass.prototype.getAttr = function (attr, target) {
        if (target === void 0) { target = this; }
        return target.getAttribute(attr);
    };
    /**
     * set attribute value
     * @param {string} attribute name
     * @param {string} new/changed value for the attribute
     * @param {object} target DOM element
     */
    BootstrapClass.prototype.setAttr = function (attr, val, target) {
        if (target === void 0) { target = this; }
        target.setAttribute(attr, val);
    };
    /**
     * remove attribute
     * @param {string} attribute name
     * @param {object} target DOM element
     */
    BootstrapClass.prototype.removeAttr = function (attr, target) {
        if (target === void 0) { target = this; }
        target.removeAttribute(attr);
    };
    /**
     * helper function to add class name using classList
     * @param {string} className string of class name
     * @param {object} target DOM element
     * @return {object} element DOM element
     */
    BootstrapClass.prototype.addClass = function (className, target) {
        if (target === void 0) { target = this; }
        target.classList.add(className);
        return target;
    };
    /**
     * helper function to remove class name using classList
     * @param {string} className string of class name
     * @param {object} target DOM element
     * @return {object} element DOM element
     */
    BootstrapClass.prototype.removeClass = function (className, target) {
        if (target === void 0) { target = this; }
        target.classList.remove(className);
        return target;
    };
    /**
     * helper function to check for has class name using classList
     * @param {string} className string of class name
     * @param {object} target DOM element
     * @return {boolean} contains true/false
     */
    BootstrapClass.prototype.hasClass = function (className, target) {
        if (target === void 0) { target = this; }
        return target.classList.contains(className);
    };
    /**
     * helper function to toggle class name using classList
     * @param {string} className string of class name
     * @param {object} target DOM element
     * @param {boolean} force boolean to use force optionally
     * @return {object} element DOM element
     */
    BootstrapClass.prototype.toggleClass = function (className, target, force) {
        if (target === void 0) { target = this; }
        return target.classList.toggle(className, force);
    };
    /**
     * helper to find child nodes rendered inside
     * @param {string} selector that is used to search the DOM node
     * @param {object} target Element in which search should be performed
     * @return {object} Element
     */
    BootstrapClass.prototype.find = function (selector, target) {
        if (target === void 0) { target = this; }
        return target.querySelector(selector);
    };
    /**
     * helper to find all child nodes rendered inside
     * @param {string} selector that is used to search the DOM node
     * @param {object} target Element in which search should be performed
     * @return {array} Elements that are matched
     */
    BootstrapClass.prototype.findAll = function (selector, target) {
        if (target === void 0) { target = this; }
        return target.querySelectorAll(selector);
    };
    /**
     * simplyfied event handler function to add events
     * @param {string} eventName event type ex: click, blur, etc.
     * @param {object} target HTML element that needs event to be added
     * @param {function | Object} callback function or Object (which has handleEvent method) that needs to be triggerd
     */
    BootstrapClass.prototype.on = function (eventName, target, callback) {
        if (target === void 0) { target = this; }
        target.addEventListener(eventName, callback, false);
    };
    /**
     * simplyfied event handler function to remove events
     * @param {string} eventName event type ex: click, blur, etc.
     * @param {object} target HTML element that needs event to be added
     * @param {function | Object} callback function or Object (which has handleEvent method) that needs to be triggerd
     */
    BootstrapClass.prototype.off = function (eventName, target, callback) {
        if (target === void 0) { target = this; }
        target.removeEventListener(eventName, callback, false);
    };
    /**
     * trigger native/custom event and pass data between components
     * @param {string} eventName custom event name
     * @param {object} target element reference on which event needs to be triggerd
     * @param {object} eventData custom event data to share
     */
    BootstrapClass.prototype.trigger = function (eventName, target, eventData) {
        if (target === void 0) { target = this; }
        var triggerEvent = !eventData
            ? new Event(eventName)
            : new CustomEvent(eventName, { detail: eventData || {} });
        target.dispatchEvent(triggerEvent);
    };
    return BootstrapClass;
}(HTMLElement));
export { BootstrapClass };
export default BootstrapClass;
//# sourceMappingURL=index.js.map